package de.asap.pak.example.commands;

import de.asap.pak.jlcint.commandapi.FieldScope;
import de.asap.pak.jlcint.commandapi.Persistent;
import de.asap.pak.jlcint.interpreter.InterpreterException;
import de.asap.pak.jlcint.testutils.JlcintExtension;
import de.asap.pak.jlcint.testutils.Provider;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;

import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.junit.jupiter.api.Assertions.assertEquals;

/**
 * Simple test class for the @{@link HelloWorld} Command
 */
@ExtendWith(JlcintExtension.class)
public class HelloWorldTest {

    /** this field will be the input for our command test, it has the same definition like in the orignial Helloworld class */
    @Persistent(mandatory = false)
    private HelloWorld.Country country = HelloWorld.Country.AUSTRALIA;

    /** this field will be set after execution of command */
    @Persistent(scope = FieldScope.WRITE_ONLY)
    private String hiMessage;
    /**
     * this will provide a virtual engine which can execute this command, make sure you have annotated this test-class wiht @ExtendWith(JlcintExtension.class)
     */
    private Provider<HelloWorld> command;

    @Test
    void testDefaultCountry() throws InterpreterException {
        assertDoesNotThrow(() -> this.command.execute());
        assertEquals(this.hiMessage, "How are ya, mate?");
    }

    @Test
    void testJapanese() throws InterpreterException {
        this.country = HelloWorld.Country.JAPANESE;
        assertDoesNotThrow(() -> this.command.execute());
        assertEquals(this.hiMessage, HelloWorld.Country.JAPANESE.sayHi());
    }

    @Test
    void testGerman() throws InterpreterException {
        this.country = HelloWorld.Country.GERMAN;
        assertDoesNotThrow(() -> this.command.execute());
        assertEquals(this.hiMessage, "Hallo Welt");
    }
}
